<?php
/**
 * Ceon URI Mapping Install/Upgrade Script - Installs/Configures Database or upgrades database from
 * an earlier version.
 *
 * @package    ceon_uri_mapping
 * @author     Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright  Copyright 2008-2009 Ceon
 * @copyright  Copyright 2003-2007 Zen Cart Development Team
 * @copyright  Portions Copyright 2003 osCommerce
 * @link       http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license    http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version    $Id: ceon_uri_mapping_auto_upgrade.php 308 2009-02-16 22:29:45Z Bob $
 */

if (is_null($version)) {
	// Module not installed! Must create tables and default configuration
	
	// Add the URI Mappings table //////////////////////////////////////////////////////////////////
	$create_uri_mappings_table_sql = "
		CREATE TABLE IF NOT EXISTS
			`" . TABLE_CEON_URI_MAPPINGS . "`
			(
			`uri` TEXT NOT NULL,
			`language_id` INT(11) UNSIGNED DEFAULT NULL,
			`current_uri` INT(1) UNSIGNED DEFAULT '0',
			`main_page` VARCHAR(45) NULL,
			`associated_db_id` INT(11) UNSIGNED DEFAULT NULL,
			`alternate_uri` VARCHAR(255) DEFAULT NULL,
			`redirection_type_code` VARCHAR(3) DEFAULT '301',
			`date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00'
			);";
	
	$create_uri_mappings_table_result = $db->Execute($create_uri_mappings_table_sql);
	
	
	// Add the configuration table /////////////////////////////////////////////////////////////////
	$create_uri_mapping_configs_table_sql = "
		CREATE TABLE IF NOT EXISTS
			`" . TABLE_CEON_URI_MAPPING_CONFIGS . "`
			(
			`id` INT UNSIGNED NOT NULL,
			`version` varchar(14) NOT NULL,
			`autogen_new` INT(1) UNSIGNED NOT NULL,
			`whitespace_replacement` INT(1) UNSIGNED NOT NULL,
			`capitalisation` INT(1) UNSIGNED NOT NULL,
			`remove_words` TEXT DEFAULT NULL,
			`char_str_replacements` TEXT DEFAULT NULL,
			PRIMARY KEY  (`id`)
		);";
	
	$create_uri_mapping_configs_table_result = $db->Execute($create_uri_mapping_configs_table_sql);
	
	
	// Set up the default configuration ////////////////////////////////////////////////////////////
	$create_default_config_sql = "
		INSERT INTO
			" . TABLE_CEON_URI_MAPPING_CONFIGS . "
			(
			id,
			version,
			autogen_new,
			whitespace_replacement,
			capitalisation,
			remove_words,
			char_str_replacements
			)
		VALUES
			(
			'1',
			'" . CEON_URI_MAPPING_VERSION . "',
			'1',
			'" . CEON_URI_MAPPING_SINGLE_UNDERSCORE . "',
			'" . CEON_URI_MAPPING_CAPITALISATION_LOWERCASE . "',
			'-, a, an, this, that',
			'=>GBP, $=>USD'
			);";
	
	$create_default_config_result = $db->Execute($create_default_config_sql);
	
	// Add the configuration group and options /////////////////////////////////////////////////////
	$add_config_group_options_sql = "
		INSERT INTO
			" . TABLE_CONFIGURATION_GROUP . "
			(
			configuration_group_title,
			configuration_group_description,
			sort_order,
			visible
			)
		VALUES
			(
			'Ceon URI Mapping (SEO)',
			'Set Ceon URI Mapping (SEO) Options',
			'1',
			'1'
			);";
	
	$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
	
	$add_config_group_options_sql = "
		SELECT
			configuration_group_id  
		FROM
			" . TABLE_CONFIGURATION_GROUP . "
		WHERE
			configuration_group_title = 'Ceon URI Mapping (SEO)';";
	
	$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
	
	if (!$add_config_group_options_result->EOF) {
		$configuration_group_id =
			$add_config_group_options_result->fields['configuration_group_id'];
		
		$add_config_group_options_sql = "
			UPDATE
				" . TABLE_CONFIGURATION_GROUP . "
			SET
				sort_order = '" . $configuration_group_id . "'
			WHERE
				configuration_group_id = '" . $configuration_group_id . "';";
		
		$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
		
		
		$add_config_group_options_sql = "
			INSERT INTO
				" . TABLE_CONFIGURATION . "
				(
				`configuration_title`,
				`configuration_key`,
				`configuration_value`,
				`configuration_description`,
				`configuration_group_id`,
				`sort_order`,
				`set_function`,
				`date_added`
				)
			VALUES
				(
				'Enable/Disable URI Mapping',
				'CEON_URI_MAPPING_ENABLED',
				'1',
				'If enabled, any Categories/Products/EZ-Pages/other pages which have a static URI Mapping specified for them in the database will use those static URIs instead of the standard Zen Cart dynamically-built URIs.<br /><br />0 = off <br />1 = on',
				'" . $configuration_group_id . "',
				'1',
				'zen_cfg_select_option(array(''0'', ''1''), ',
				NOW()
				);";
		
		$add_config_group_options_result = $db->Execute($add_config_group_options_sql);
	}
	
	$ceon_uri_mapping_just_installed = true;
} else if (substr($version, 0, 1) == '2') {
	// Must update the database!
	
	// Add the new current uri column to the region configs table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`current_uri` INT(1) UNSIGNED DEFAULT '0'
		AFTER
			`language_id`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new main page column to the region configs table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`main_page` VARCHAR(45) NULL
		AFTER
			`current_uri`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new associated DB ID column to the region configs table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`associated_db_id` INT(11) UNSIGNED DEFAULT NULL
		AFTER
			`main_page`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new redirection type code column to the region configs table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`redirection_type_code` VARCHAR(3) DEFAULT '301'
		AFTER
			`alternate_uri`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	// Add the new date added column to the region configs table
	$add_column_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		ADD
			`date_added` DATETIME NOT NULL DEFAULT '0000-00-00 00:00:00'
		AFTER
			`redirection_type_code`;";
	
	$add_column_result = $db->Execute($add_column_sql);
	
	
	// Now modify all existing URI Mappings to use the new format //////////////////////////////////
	$uri_mappings_query = "
		SELECT
			um.*,
			pt.type_handler
		FROM
			" . TABLE_CEON_URI_MAPPINGS . " um
		LEFT JOIN
			" . TABLE_PRODUCTS . " p
		ON
			p.products_id = um.product_id
		LEFT JOIN
			" . TABLE_PRODUCT_TYPES . " pt
		ON
			pt.type_id = p.products_type
		WHERE
			1 = 1;";
	
	$uri_mappings_result = $db->Execute($uri_mappings_query);
	
	while (!$uri_mappings_result->EOF) {
		$main_page = null;
		
		if (!is_null($uri_mappings_result->fields['category_id'])) {
			$main_page = FILENAME_DEFAULT;
			$associated_db_id = $uri_mappings_result->fields['category_id'];
			
			$selection_query_part = "category_id = '" .
				$uri_mappings_result->fields['category_id'] . "'";
		} else if (!is_null($uri_mappings_result->fields['product_id'])) {
			$main_page = $uri_mappings_result->fields['type_handler'] . '_info';
			$associated_db_id = $uri_mappings_result->fields['product_id'];
			
			$selection_query_part = "product_id = '" .
				$uri_mappings_result->fields['product_id'] . "'";
		} else if (!is_null($uri_mappings_result->fields['page_id'])) {
			$main_page = FILENAME_EZPAGES;
			$associated_db_id = $uri_mappings_result->fields['page_id'];
			
			$selection_query_part = "page_id = '" .
				$uri_mappings_result->fields['page_id'] . "'";
		}
		
		if (!is_null($main_page)) {
			$update_record_format_query = "
				UPDATE
					" . TABLE_CEON_URI_MAPPINGS . "
				SET
					current_uri = '1',
					main_page = '" . zen_db_input($main_page) . "',
					associated_db_id = '" . zen_db_input($associated_db_id) . "'
				WHERE
					" . $selection_query_part . ";";
			
			$update_record_format_result = $db->Execute($update_record_format_query);
			
			// Must add the default URI mappings for the other product-related pages
			$uri_mapping = $uri_mappings_result->fields['uri'];
			
			if (substr($uri_mapping, -1) != '/') {
				$uri_mapping .= '/';
			}
			
			$sql_data_array = array(
				'language_id' => $uri_mappings_result->fields['language_id'],
				'current_uri' => 1,
				'associated_db_id' => $associated_db_id,
				'alternate_uri' => (!is_null($uri_mappings_result->fields['alternate_uri']) ?
					$uri_mappings_result->fields['alternate_uri'] : 'null'),
				'redirection_type_code' => $uri_mappings_result->fields['redirection_type_code'],
				'date_added' => $uri_mappings_result->fields['date_added']
				);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS);
			$sql_data_array['main_page'] = zen_db_prepare_input(FILENAME_PRODUCT_REVIEWS);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS_INFO);
			$sql_data_array['main_page'] =
				zen_db_prepare_input(FILENAME_PRODUCT_REVIEWS_INFO);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_PRODUCT_REVIEWS_WRITE);
			$sql_data_array['main_page'] =
				zen_db_prepare_input(FILENAME_PRODUCT_REVIEWS_WRITE);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
			
			$sql_data_array['uri'] = zen_db_prepare_input($uri_mapping .
				CEON_URI_MAPPING_URI_PART_TELL_A_FRIEND);
			$sql_data_array['main_page'] =
				zen_db_prepare_input(FILENAME_TELL_A_FRIEND);
			
			zen_db_perform(TABLE_CEON_URI_MAPPINGS, $sql_data_array);
		}
		
		$uri_mappings_result->MoveNext();
	}
	
	// Update the date columns to give them a default date
	$update_date_added_query = "
		UPDATE
			" . TABLE_CEON_URI_MAPPINGS . "
		SET
			date_added = NOW()
		WHERE
			1 = 1;";
	
	$update_date_added_result = $db->Execute($update_date_added_query);
	
	// Set the alternate_uri column's values to null where appropriate
	$update_alternate_uri_query = "
		UPDATE
			" . TABLE_CEON_URI_MAPPINGS . "
		SET
			alternate_uri = NULL
		WHERE
			alternate_uri = 'null';";
	
	$update_alternate_uri_result = $db->Execute($update_alternate_uri_query);
	
	// Remove the old columns
	$drop_columns_sql = "
		ALTER TABLE
			" . TABLE_CEON_URI_MAPPINGS . "
		DROP 
			`category_id`,
		DROP 
			`product_id`,
		DROP 
			`page_id`;";
	
	$drop_columns_result = $db->Execute($drop_columns_sql);
} else if (substr($version, 0, 1) == '3') {
	// No changes necessary
}


// Update version number of database tables ////////////////////////////////////////////////////////
// Only one config currently supported so ID is hard-coded in following SQL
$update_db_version_number_sql = "
	UPDATE
		" . TABLE_CEON_URI_MAPPING_CONFIGS . "
	SET
		version = '" . CEON_URI_MAPPING_VERSION . "'
	WHERE
		id = '1';";

$update_db_version_number_result = $db->Execute($update_db_version_number_sql);

// Finished!
if (!isset($ceon_uri_mapping_just_installed)) {
	$ceon_uri_mapping_upgraded = true;
}

?>