<?php
/**
 * Ceon URI Mapping functionality initialisation script
 * 
 * @package    ceon_uri_mapping
 * @author     Conor Kerr <zen-cart.uri_mapping@dev.ceon.net>
 * @copyright  Copyright 2008-2009 Ceon
 * @copyright  Copyright 2003-2007 Zen Cart Development Team
 * @copyright  Portions Copyright 2003 osCommerce
 * @link       http://dev.ceon.net/web/zen-cart/uri_mapping
 * @license    http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version    $Id: init_ceon_uri_mapping.php 308 2009-02-16 22:29:45Z Bob $
 */

if (!defined('IS_ADMIN_FLAG')) {
	die('Illegal Access');
}

if (defined('CEON_URI_MAPPING_ENABLED') && CEON_URI_MAPPING_ENABLED == 1) {
	// Run the language initsystem code here as, for some strange reason, it runs late in the
	// initsystem process and the Ceon URI Mapping module must run before the sanitize initsystem
	// script which normally preceeds the language initsystem code.
	if (!isset($_SESSION['language']) || isset($_GET['language'])) {
		$lng = new language();
		if (isset($_GET['language']) && zen_not_null($_GET['language'])) {
			$lng->set_language($_GET['language']);
		} else {
			if (LANGUAGE_DEFAULT_SELECTOR=='Browser') {
				$lng->get_browser_language();
			} else {
				$lng->set_language(DEFAULT_LANGUAGE);
			}
		}
		$_SESSION['language'] = (zen_not_null($lng->language['directory']) ? $lng->language['directory'] : 'english');
		$_SESSION['languages_id'] = (zen_not_null($lng->language['id']) ? $lng->language['id'] : 1);
		$_SESSION['languages_code'] = (zen_not_null($lng->language['code']) ? $lng->language['code'] : 'en');
	}
	if (!isset($_GET['main_page']) && isset($_SERVER['REQUEST_URI']) &&
			$_SERVER['REQUEST_URI'] != '/') {
		// Standard Zen Cart URI isn't being used, attempt to identify and map URI.
		
		// Get the URI
		$request_uri = $_SERVER['REQUEST_URI'];
		
		// Remove any query string
		$query_pos = strpos($request_uri, '?');
		if ($query_pos !== false) {
			$request_uri = substr($request_uri, 0, $query_pos);
		}
		
		// Attempt to match slight variations of the URI by matching against a regular expression that
		// allows for an optional trailing slash
		// First make sure that request URI has no invalid characters in it
		$pattern = preg_replace('/[^a-zA-Z0-9_\-\.\/]/', '', $request_uri);
		
		if (substr($pattern, -1) == '/') {
			// Remove the trailing slash to prevent duplication in the regular expression
			$pattern = substr($pattern, 0, strlen($pattern) -1);
		}
		
		$pattern = '^' . $pattern . '\/?$';
		
		$identify_main_page_query = "
			SELECT
				um.language_id,
				um.main_page,
				um.associated_db_id,
				um.alternate_uri,
				um.current_uri,
				um.redirection_type_code
			FROM
				" . TABLE_CEON_URI_MAPPINGS . " um
			WHERE
				um.uri REGEXP '" . zen_db_prepare_input($pattern) . "'
			ORDER BY
				language_id,
				current_uri DESC,
				date_added DESC;";
		
		$identify_main_page_result = $db->Execute($identify_main_page_query);
		
		if (!$identify_main_page_result->EOF) {
			// Have matched the URI!
			
			// Use the mapping which specifically maps the customer's language ID (if possible),
			// otherwise simply use the last match
			while (!$identify_main_page_result->EOF) {
				$language_id = $identify_main_page_result->fields['language_id'];
				$main_page = $identify_main_page_result->fields['main_page'];
				$associated_db_id = $identify_main_page_result->fields['associated_db_id'];
				$alternate_uri = $identify_main_page_result->fields['alternate_uri'];
				$current_uri = $identify_main_page_result->fields['current_uri'];
				$redirection_type_code = $identify_main_page_result->fields['redirection_type_code'];
				
				if ($language_id == $_SESSION['languages_id']) {
					break;
				}
				
				$identify_main_page_result->MoveNext();
			}
			
			if (!is_null($alternate_uri)) {
				$alternate_uri = trim($alternate_uri);
				if (strlen($alternate_uri) < 2) {
					$alternate_uri = null;
				}
			}
			
			if (!is_null($main_page) && is_null($alternate_uri)) {
				if ($current_uri != 1) {
					// This isn't a current URI, is there a current URI to redirect to? Or is this
					// an old URI being kept in the history to avoid broken links?
					if (!is_null($associated_db_id)) {
						$redirection_uri_query = "
							SELECT
								um.uri
							FROM
								" . TABLE_CEON_URI_MAPPINGS . " um
							WHERE
								um.main_page = '" . zen_db_input(zen_db_prepare_input($main_page)) . "'
							AND
								um.associated_db_id = '" . (int) $associated_db_id . "'
							AND
								um.language_id = '" . (int) $_SESSION['languages_id'] . "'
							AND
								current_uri = '1';";
					} else {
						$redirection_uri_query = "
							SELECT
								um.uri
							FROM
								" . TABLE_CEON_URI_MAPPINGS . " um
							WHERE
								um.main_page = '" . zen_db_input(zen_db_prepare_input($main_page)) . "'
							AND
								um.language_id = '" . (int) $_SESSION['languages_id'] . "'
							AND
								current_uri = '1';";
					}
					
					$redirection_uri_result = $db->Execute($redirection_uri_query);
					
					if (!$redirection_uri_result->EOF) {
						// Have found a current URI mapping for this page, force a redirect to it
						$redirection_uri = $redirection_uri_result->fields['uri'];
						
						// Include any extra info in the GET variables
						$query_string = '';
						foreach ($_GET as $key => $value) {
							$query_string .= '&' . urlencode($key) . '=' . urlencode($value);
						}
						
						if ($query_string != '') {
							$query_string = '?' . substr($query_string, 1, strlen($query_string) - 1);
						}
						
						header("HTTP/1.1 301 Moved Permanently");
						header("Location: " . $redirection_uri . $query_string);
						zen_exit();
					} else {
						// There is no current URI, this is a URI from the historical database being
						// used to prevent broken links. Force a redirect to standard Zen Cart page
						
						if (!is_null($associated_db_id)) {
							// Must supply the ID information necessary for the correct info to be
							// loaded
							switch ($main_page) {
								case FILENAME_DEFAULT:
									if (!isset($_GET['cPath'])) {
										$_GET['cPath'] = $associated_db_id;
									}
									break;
								case FILENAME_EZPAGES:
								case FILENAME_EZPAGES_POPUP:
									$_GET['id'] = $associated_db_id;
									break;
								default:
									// This is a product related page
									$_GET['products_id'] = $associated_db_id;
									break;
							}
						}
						
						if ($request_type == 'SSL') {
							$redirection_uri = DIR_WS_HTTPS_CATALOG;
						} else {
							$redirection_uri = DIR_WS_CATALOG;
						}
						
						$redirection_uri .= 'index.php?main_page=' . $main_page;
						$parameters = zen_get_all_get_params();
						
						if (strlen($parameters) > 0) {
							$redirection_uri .= '&' . $parameters;
						}
						
						if (substr($redirection_uri, 0, 1) != '/') {
							$redirection_uri = '/' . $redirection_uri;
						}
						
						if (substr($redirection_uri, -1) == '&') {
							$redirection_uri = substr($redirection_uri, 0,
								strlen($redirection_uri) - 1);
						}
						
						// Record that the source of redirect was this script, avoiding any
						// unnecessary processing later
						$_SESSION['ceon_uri_mapping_redirected'] = true;
						
						header("HTTP/1.1 301 Moved Permanently");
						header("Location: " . $redirection_uri);
						zen_exit();
					}
				}
				
				// This URI is the current URI for this page
				$_GET['main_page'] = $main_page;
				
				if (!is_null($associated_db_id)) {
					// Page is a category, EZ-Page or a product info page, must initialise the
					// appropriate ID
					if ($main_page == FILENAME_DEFAULT) {
						// This is a category page
						// Must build the full contextual link to the category, so any hierarchy
						// is respected
						$categories = array();
						zen_get_parent_categories($categories, $associated_db_id);
						
						$categories = array_reverse($categories);
						
						$cPath = implode('_', $categories);
						
						if (zen_not_null($cPath)) {
							$cPath .= '_';
						}
						$cPath .= $associated_db_id;
						
						$_GET['cPath'] = $cPath;
					} else if ($main_page == FILENAME_EZPAGES ||
							$main_page == FILENAME_EZPAGES_POPUP) {
						// Have found an EZ-Page. Should it be displayed as a normal page or in a
						// popup?
						if ($main_page == FILENAME_EZPAGES_POPUP || isset($_GET['ezpopup'])) {
							$_GET['main_page'] = FILENAME_EZPAGES_POPUP;
						} else {
							$_GET['main_page'] = FILENAME_EZPAGES;
						}
						$_GET['id'] = $associated_db_id;
					} else {
						// This is a product related page
						$_GET['products_id'] = $associated_db_id;
					}
				}
			} else if (!is_null($alternate_uri)) {
				// Have matched an alternative URI, is this a Zen Cart page with preset parameters
				// or a full URI which must be redirected to?
				$pattern = '/index\.php\?main_page\=([^\&]+)\&?(.*)/';
				
				if (preg_match($pattern, $alternate_uri, $matches)) {
					// Alternative URI is a Zen Cart page, get any parameters to be set
					$_GET['main_page'] = $matches[1];
					$parameter_string = $matches[2];
					
					$parameters = explode('&', $parameter_string);
					
					$num_parameters = sizeof($parameters);
					for ($i = 0; $i < $num_parameters; $i++) {
						$current_parameter_pair = explode('=', $parameters[$i]);
						if (sizeof($current_parameter_pair) == 2) {
							$_GET[$current_parameter_pair[0]] = $current_parameter_pair[1];
						}
					}
				} else {
					// Alternative URI is a full URI to be redirected to
					switch ($redirection_type_code) {
						case 301:
							header("HTTP/1.1 301 Moved Permanently");
							break;
						case 302:
							header("HTTP/1.1 302 Found");
							break;
						case 303:
							header("HTTP/1.1 302 See Other ");
							break;
						case 307:
							header("HTTP/1.1 302 Temporary Redirect");
							break;
					}
					header("Location: " . $alternate_uri);
					zen_exit();
				}
			} else {
				print "Unable to map identified URI: $request_uri";
			}
		}
	} else if (isset($_GET['main_page']) && isset($_SESSION['ceon_uri_mapping_redirected'])) {
		// This script redirected the customer here so there's no need for any further checks
		unset($_SESSION['ceon_uri_mapping_redirected']);
	} else if (isset($_GET['main_page']) && !isset($_SESSION['ceon_uri_mapping_redirected'])) {
		// This is a standard Zen Cart dynamic URI - redirect it to its mapped URI if there is one
		$associated_db_id = null;
		
		if ($_GET['main_page'] == FILENAME_DEFAULT) {
			// Is this a category?
			if (isset($_GET['cPath'])) {
				// This is a category page - get the URI for the category ID
				$category_parts = explode('_', $_GET['cPath']);
				$category_id = $category_parts[sizeof($category_parts) - 1];
				
				$associated_db_id = $category_id;
			}
		} else if($_GET['main_page'] == FILENAME_DOCUMENT_GENERAL_INFO ||
				$_GET['main_page'] == FILENAME_DOCUMENT_PRODUCT_INFO ||
				$_GET['main_page'] == FILENAME_PRODUCT_INFO ||
				$_GET['main_page'] == FILENAME_PRODUCT_BOOK_INFO ||
				$_GET['main_page'] == FILENAME_PRODUCT_FREE_SHIPPING_INFO ||
				$_GET['main_page'] == FILENAME_PRODUCT_MUSIC_INFO ||
				$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS ||
				$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_INFO ||
				$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_WRITE ||
				$_GET['main_page'] == FILENAME_TELL_A_FRIEND) {
			$associated_db_id = $_GET['products_id'];
		} else if ($_GET['main_page'] == FILENAME_EZPAGES ||
				$_GET['main_page'] == FILENAME_EZPAGES_POPUP) {
			$associated_db_id = $_GET['id'];
		}
		
		if (!is_null($associated_db_id)) {
			$redirection_uri_query = "
				SELECT
					um.uri
				FROM
					" . TABLE_CEON_URI_MAPPINGS . " um
				WHERE
					um.main_page = '" . zen_db_input(zen_db_prepare_input($_GET['main_page'])) . "'
				AND
					um.associated_db_id = '" . (int) $associated_db_id . "'
				AND
					um.language_id = '" . (int) $_SESSION['languages_id'] . "'
				AND
					current_uri = '1';";
		} else {
			$redirection_uri_query = "
				SELECT
					um.uri
				FROM
					" . TABLE_CEON_URI_MAPPINGS . " um
				WHERE
					um.main_page = '" . zen_db_input(zen_db_prepare_input($_GET['main_page'])) . "'
				AND
					um.language_id = '" . (int) $_SESSION['languages_id'] . "'
				AND
					current_uri = '1';";
		}
		
		$redirection_uri_result = $db->Execute($redirection_uri_query);
		
		if (!$redirection_uri_result->EOF) {
			// Have found a URI mapping for this page, use this as the "current" URI for this page
			// by forcing a redirect to it
			$redirection_uri = $redirection_uri_result->fields['uri'];
			
			// Include any extra info in the GET variables
			if ($_GET['main_page'] == FILENAME_DEFAULT && isset($_GET['cPath'])) {
				// Remove the category ID from the URL as it will be regenerated when the static
				// URI is loaded
				unset($_GET['cPath']);
			} else if($_GET['main_page'] == FILENAME_DOCUMENT_GENERAL_INFO ||
					$_GET['main_page'] == FILENAME_DOCUMENT_PRODUCT_INFO ||
					$_GET['main_page'] == FILENAME_PRODUCT_INFO ||
					$_GET['main_page'] == FILENAME_PRODUCT_BOOK_INFO ||
					$_GET['main_page'] == FILENAME_PRODUCT_FREE_SHIPPING_INFO ||
					$_GET['main_page'] == FILENAME_PRODUCT_MUSIC_INFO ||
					$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS ||
					$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_INFO ||
					$_GET['main_page'] == FILENAME_PRODUCT_REVIEWS_WRITE ||
					$_GET['main_page'] == FILENAME_TELL_A_FRIEND) {
				unset($_GET['products_id']);
			} else  if ($_GET['main_page'] == FILENAME_EZPAGES ||
					$_GET['main_page'] == FILENAME_EZPAGES_POPUP) {
				unset($_GET['id']);
			}
			
			unset($_GET['main_page']);
			
			$query_string = '';
			foreach ($_GET as $key => $value) {
				$query_string .= '&' . urlencode($key) . '=' . urlencode($value);
			}
			
			if ($query_string != '') {
				$query_string = '?' . substr($query_string, 1, strlen($query_string) - 1);
			}
			
			header("HTTP/1.1 301 Moved Permanently");
			header("Location: " . $redirection_uri . $query_string);
			zen_exit();
		}
	} else if (!isset($_SERVER['REQUEST_URI'])) {
		print 'Ceon URI Mapping cannot access REQUEST_URI server variable!';
	}
}

?>